/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.cluster;

import org.apache.log4j.Logger;
import org.jgroups.util.Streamable;

import com.koutra.dist.proc.model.IPipelineItemTemplate;
import com.koutra.dist.proc.model.ISimplePipelineItemTemplate;
import com.koutra.dist.proc.model.XformationException;
import com.koutra.dist.proc.pipeline.AbstractTaskBoundaryTemplate;

/**
 * Concrete implementation of the <code>IClusterRunnable</code> interface.
 * 
 * @author Pafsanias Ftakas
 */
public class ClusterRunnable implements IClusterRunnable {
	private static final Logger logger = Logger.getLogger(ClusterRunnable.class);
	
	protected Streamable wirePayload;
	private AbstractTaskBoundaryTemplate thisPtr;
	
	/**
	 * Initializing constructor. Creates a cluster runnable for the specified
	 * <code>Streamable</code> instance.
	 * @param wirePayload
	 */
	public ClusterRunnable(Streamable wirePayload) {
		this.wirePayload = wirePayload;
		setConainingObject(wirePayload);
	}

	/**
	 * Implementation of the <code>IClusterRunnable</code> interface.
	 */
	@Override
	public Streamable getWirePayload() {
		return wirePayload;
	}
	
	/**
	 * Implementation of the <code>IClusterRunnable</code> interface.
	 */
	@Override
	public void setConainingObject(Object container) {
		thisPtr = (AbstractTaskBoundaryTemplate) container;
	}
	
	/**
	 * Implementation of the <code>IClusterRunnable</code> interface.
	 */
	@Override
	public void run() {
		if (logger.isTraceEnabled())
			logger.trace("Driving pipeline from boundary to boundary");
		
		// Go down the sink pipeline until you read the task boundary. There
		// call the consume interface to get the whole task to execute
		// in the cluster.
		IPipelineItemTemplate template = (IPipelineItemTemplate) thisPtr.getSink();
		while (!(template instanceof AbstractTaskBoundaryTemplate)) {
			if (template instanceof ISimplePipelineItemTemplate)
				template = (IPipelineItemTemplate)
						((ISimplePipelineItemTemplate) template).getSink();
			else
				throw new XformationException("Unexpected template type: " +
						template.getClass().getCanonicalName());
		}
		AbstractTaskBoundaryTemplate boundaryTemplate =
			(AbstractTaskBoundaryTemplate) template;
		
		boundaryTemplate.consume(null);
		
		if (logger.isTraceEnabled())
			logger.trace("Executable for the task boundary " +
					"(end of pipeline) will now exit.");
	}

}
