/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.pipeline.script;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.script.Invocable;
import javax.script.ScriptEngine;
import javax.script.ScriptException;

import org.apache.log4j.Logger;

import com.koutra.dist.proc.util.EngineHelper;

public class ReaderToWriterPipelineItem extends
		com.koutra.dist.proc.pipeline.ReaderToWriterPipelineItem {
	private static final Logger logger = Logger.getLogger(
			ReaderToWriterPipelineItem.class);
	
	protected String scriptEngineName;
	protected String scriptPath;
	
	/**
	 * @deprecated use one of the initializing constructors
	 */
	public ReaderToWriterPipelineItem() { }
	
	public ReaderToWriterPipelineItem(String id, String scriptEngineName, String scriptPath) {
		super(id);
		this.scriptEngineName = scriptEngineName;
		this.scriptPath = scriptPath;
	}

	@Override
	public char[] transformBuffer(char[] buffer, int off, int len) {
		try {
			ScriptEngine scriptEngine = EngineHelper.getInstance().evalScript(
					scriptEngineName, scriptPath);
			Invocable invocableEngine = (Invocable) scriptEngine;
			return (char[]) invocableEngine.invokeFunction(
					"transformBuffer", buffer, off, len);
		} catch (ScriptException e) {
			logger.error("Error invoking function 'transformBuffer'", e);
		} catch (NoSuchMethodException e) {
			logger.error("Error invoking function 'transformBuffer'", e);
		} catch (FileNotFoundException e) {
			logger.error("Error invoking function 'transformBuffer'", e);
		}
		
		return new char[0];
	}
	
	@Override
	public void readFrom(DataInputStream in) throws IOException,
			IllegalAccessException, InstantiationException {
		super.readFrom(in);
		scriptEngineName = in.readUTF();
		scriptPath = in.readUTF();
	}
	
	@Override
	public void writeTo(DataOutputStream out) throws IOException {
		super.writeTo(out);
		out.writeUTF(scriptEngineName);
		out.writeUTF(scriptPath);
	}

}
