/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.sax;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class StartElementEvent extends SAXEvent {
	private static final AttributesImpl emptyAttributes = new AttributesImpl();
	
	protected String uri;
	protected String localName;
	protected String qName;
	protected Attributes atts;
	
	public StartElementEvent(String uri, String localName, String qName,
			Attributes atts) {
		this.uri = uri;
		this.localName = localName;
		this.qName = qName;
		this.atts = atts.getLength() == 0 ? emptyAttributes : new AttributesImpl(atts);
	}

	@Override
	public void fireEvent(ContentHandler contentHandler) throws SAXException {
		contentHandler.startElement(uri, localName, qName, atts);
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < atts.getLength(); i++) {
			if (sb.length() != 0)
				sb.append(",");
			sb.append(atts.getLocalName(i));
			sb.append("=");
			sb.append(atts.getValue(i));
		}
		return "StartElement[" + localName + "," + qName + ",{" + sb.toString() + "}]";
	}

}
