/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.xml;

import java.util.Stack;

public class ElementStackPath {
	
	private String pathString;
	private Stack<Integer> prevPos = new Stack<Integer>();
	
	public ElementStackPath() {
		this.pathString = null;
		this.prevPos = new Stack<Integer>();
	}

	public void startDocument() {
		pathString = "/";
	}
	
	public void push(String eName) {
		prevPos.push(new Integer(pathString.length()));
		
		String potentialSeparator = "";
		if (pathString.charAt(pathString.length() - 1) != '/')
			potentialSeparator = "/";
		
		pathString += potentialSeparator + eName;
	}
	
	public void pop() {
		int length = ((Integer) prevPos.pop()).intValue();
		pathString = pathString.substring(0, length);
	}
	

	private boolean compareTo(String xPath, int start, int length) {
		if (xPath.length() != length)
			return false;
		
		for (int i=start; i<length; i++) {
			if (xPath.charAt(i) != pathString.charAt(i))
				return false;
		}
		
		return true;
	}
	
	public boolean compareTo(String xPath, int matchingLevel) {
		int length = 0;
		if (matchingLevel <= 0 || matchingLevel >= prevPos.size())
			length = pathString.length();
		else
			length = ((Integer) prevPos.get(matchingLevel)).intValue();
		
		return compareTo(xPath, 0, length);
	}
	
	public String toString() {
		return pathString;
	}

}
