/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.xml;

import java.util.LinkedList;
import java.util.concurrent.BlockingDeque;
import java.util.concurrent.LinkedBlockingDeque;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import com.koutra.dist.proc.model.XformationException;
import com.koutra.dist.proc.sax.CharactersEvent;
import com.koutra.dist.proc.sax.EndDocumentEvent;
import com.koutra.dist.proc.sax.EndElementEvent;
import com.koutra.dist.proc.sax.SAXEvent;
import com.koutra.dist.proc.sax.StartDocumentEvent;
import com.koutra.dist.proc.sax.StartElementEvent;

public class XMLDemuxContentHandler implements ContentHandler {
	private static final Logger logger = Logger.getLogger(XMLDemuxContentHandler.class);
	
	protected BlockingDeque<LinkedList<SAXEvent>> deque;
	protected LinkedList<SAXEvent> eventList;
	
	public XMLDemuxContentHandler() {
		deque = new LinkedBlockingDeque<LinkedList<SAXEvent>>(100);
		eventList = new LinkedList<SAXEvent>();
	}
	
	public LinkedList<SAXEvent> getNextEventList() throws InterruptedException {
		LinkedList<SAXEvent> eventList = deque.takeFirst();
		if (logger.isTraceEnabled()) {
			StringBuilder sb = new StringBuilder("[");
			for (SAXEvent event : eventList) {
				if (sb.length() != 1)
					sb.append(",");
				sb.append(event.toString());
			}
			sb.append("]");
			logger.trace("Next event list: " + sb.toString());
		}
		return eventList;
	}
	
	protected void switchEventList() {
		if (!(eventList.getLast() instanceof EndDocumentEvent) ||
				!(eventList.getFirst() instanceof StartDocumentEvent))
				throw new XformationException(
						"SAX event list not matching a full document");
		
		deque.addLast(eventList);
		eventList = new LinkedList<SAXEvent>();
	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		eventList.add(new CharactersEvent(ch, start, length));
	}

	@Override
	public void startDocument() throws SAXException {
		eventList.add(new StartDocumentEvent());
	}

	@Override
	public void startElement(String uri, String localName, String name,
			Attributes atts) throws SAXException {
		eventList.add(new StartElementEvent(uri, localName, name, atts));
	}

	@Override
	public void endDocument() throws SAXException {
		eventList.add(new EndDocumentEvent());
		
		switchEventList();
	}

	@Override
	public void endElement(String uri, String localName, String name)
			throws SAXException {
		eventList.add(new EndElementEvent(uri, localName, name));
	}

	//////////////////////////////////////////////////////////////////////////////////////
	
	@Override
	public void endPrefixMapping(String prefix) throws SAXException {
		// Intentionally left empty
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
		// Intentionally left empty
	}

	@Override
	public void processingInstruction(String target, String data)
			throws SAXException {
		// Intentionally left empty
	}

	@Override
	public void setDocumentLocator(Locator locator) {
		// Intentionally left empty
	}

	@Override
	public void skippedEntity(String name) throws SAXException {
		// Intentionally left empty
	}

	@Override
	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
		// Intentionally left empty
	}
}
